package graphoutput;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;      
import javax.media.j3d.*;
import javax.vecmath.*;



/**
 * MarkerDlg - Dialog to choose a color by clicking
 * on a coloured button
 */
class MarkerDlg extends Dialog
implements WindowListener, ActionListener
{                            
  Frame parent;
  Object marker;
  
  TextField tf_time, tf_trans;   
  Button col;

  public MarkerDlg(Frame parent, Object marker)
  {
	super( parent, "Specify Highlighting marker", true );
    this.parent = parent;
    this.marker = marker;

    if(!(marker instanceof MarkerEPC ||
         marker instanceof MarkerLWW ||
         marker instanceof MarkerTDC) )
    {
      setVisible(false);
      dispose();
    }

    createComponents();

    pack();
    addWindowListener(this);
    setVisible(true);
  }


  public void actionPerformed(ActionEvent e)
  {
	if(e.getActionCommand().equals("Change"))
	{
      new ColorDlg(this, (Button)e.getSource());
	}
	if(e.getActionCommand().equals("Cancel"))
	{
      setVisible(false);
      dispose();
	}
	if(e.getActionCommand().equals("OK"))
	{
	  updateMarker();
      setVisible(false);
      dispose();
	}
  }


  // writes the information from the components into
  // this.marker
  private void updateMarker()
  {

    Color c = col.getBackground();
    Color3f c3f = new Color3f( ((float)c.getRed())/255.0f,
                               ((float)c.getGreen())/255.0f,
                               ((float)c.getBlue())/255.0f );

    float trans = (float) (new Float(tf_trans.getText())).floatValue();

    double pos = (double) (new Double(tf_time.getText())).doubleValue();

    if (marker instanceof MarkerEPC)
    {
      ((MarkerEPC)marker).cp.setColor(c3f, trans);
      ((MarkerEPC)marker).setTime(pos);
      ((OptionsDlg)parent).updateMarkerList();
    }
    else if (marker instanceof MarkerLWW)
    {
      ((MarkerLWW)marker).cp.setColor(c3f, trans);
      ((MarkerLWW)marker).setTime(pos);
      ((OptionsDlg)parent).updateMarkerList();
    }
    else if (marker instanceof MarkerTDC)
    {
      ((MarkerTDC)marker).handle.setColor(c3f, trans);
      ((MarkerTDC)marker).setTime(pos);
      ((OptionsDlg)parent).updateMarkerList();
    }


  }

  private void createComponents()
  {
    setBackground(Color.white);
    setLayout(new GridLayout(4,1));


    double endTime = 0.0;
    double curTime = 0.0;
    Color3f  c = new Color3f(0.0f, 0.0f, 0.0f);
    float    t = 0.0f;

    if (marker instanceof MarkerEPC)
    {
     endTime = ((MarkerEPC)marker).parent.time.getMax3();
     curTime = ((MarkerEPC)marker).getTime();
     c = ((MarkerEPC)marker).cp.getColor();
     t = ((MarkerEPC)marker).cp.getTransparency();
    }
    else if (marker instanceof MarkerLWW)
    {
     endTime = ((MarkerLWW)marker).parent.time.getMax3();
     curTime = ((MarkerLWW)marker).getTime();
     c = ((MarkerLWW)marker).cp.getColor();
     t = ((MarkerLWW)marker).cp.getTransparency();
    }
    else if (marker instanceof MarkerTDC)
    {
     endTime = ((MarkerTDC)marker).parent.time.getMax3();
     curTime = ((MarkerTDC)marker).getTime();
     c = ((MarkerTDC)marker).handle.getColor();
     t = ((MarkerTDC)marker).handle.getTransparency();
    }

// extend
    Panel p1 = new Panel();
    p1.add(new Label("Highlight dataset at time: (0 to "+endTime+"):", Label.RIGHT));
    tf_time = new TextField(""+curTime,6);
    p1.add(tf_time);
    add(p1);

    Panel p2 = new Panel();
    p2.add(new Label("Color:", Label.RIGHT));
    col = new Button("Change");
    col.addActionListener(this);
    col.setBackground(new Color(c.x, c.y, c.z));
    p2.add(col);
    add(p2);

    Panel p3 = new Panel();
    p3.add(new Label("Transparency:", Label.RIGHT));
    tf_trans = new TextField(""+t, 6);
    p3.add(tf_trans);
    add(p3);

    Panel p4 = new Panel();
    Button b1 = new Button("OK");
    b1.addActionListener(this);
    p4.add(b1);
    Button b2 = new Button("Cancel");
    b2.addActionListener(this);
    p4.add(b2);
    add(p4);

    pack();
  }




    public void windowActivated(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowClosing(WindowEvent e)
	{
      setVisible(false);
	  dispose();
	}
    public void windowDeactivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e) {}


}

