package graphoutput;

import javax.media.j3d.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.behaviors.picking.*;
import javax.vecmath.*;
import java.util.*;
import java.awt.*;

/**
 * This class contains the implementation of the method
 * "Three-dimensional Parallel Coordinates". It creates the geometry within
 * this BranchGroup into the 1x1x1-box.
 */
public class TDCGraph extends BranchGroup
{
    private static final int TRAJECTORY_THICKNESS = 2;
    private static final float PLANE_TRANSPARENCY = 0.5f;

    private int dimcnt, pointcnt;
	private Vector lineShape_v;
    private Vector surface_v;
	private Style style;
	private GraphOutput parent;
    private BranchGroup stripe_bg;

    /**
     * Removes the current geometry (stripes & lines)
     */
    public void removeGeometry()
    {
      stripe_bg.detach();
      surface_v = new Vector();
      lineShape_v = new Vector();
    }

    /**
     * Draws new stripes using the specified data
     */
    public void updateGeometry(Vector points_v, Vector scaling_v)
    {
      stripe_bg = new BranchGroup();
      stripe_bg.setCapability(BranchGroup.ALLOW_DETACH);

      createGeometries(dimcnt, pointcnt, points_v, scaling_v);

      addChild(stripe_bg);
      style.resetMarkers();

    }


    /**
     * Fills the stripeShape_v with the stripes as Shape3Ds
     */
	private void createGeometries(int dimcnt, int pointcnt, Vector points_v, Vector scaling_v)
	{

      double[] plane_coords = {
         0.0, 0.0, 0.0,  1.0, 0.0, 0.0,  1.0, 0.0, 1.0,  0.0, 0.0, 1.0 };
      float[] plane_normals1 = { 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f };

      Vector line_v = new Vector(); // of LineStripArray for trajectories
      lineShape_v = new Vector(); // of Shape3D
      surface_v = new Vector(); // the connecting surfaces
      LineStripArray conn_lines;
      double space=1.0;             // vert. space between planes
      int[] c = { pointcnt };
      int[] four = { 4 };

      // hide the coordinate-box
      parent.hideCoordinateBox();

	  int plane_cnt = (1+dimcnt)/2;
      if(plane_cnt>1)
        space = 1 / (double)(plane_cnt-1);
      else space = 1.0;

      // for each plane
      for(int j=0; j<plane_cnt; j++)
      {
        // init all LineStripArrays and add to Vector
        line_v.addElement(
		  new LineStripArray(pointcnt, LineArray.COORDINATES, c));

	    // draw the plane
        if(style.showplanes==true)
        {
          TriangleFanArray tfa1 = new TriangleFanArray(
            4, GeometryArray.COORDINATES | GeometryArray.NORMALS, four);
          tfa1.setCoordinates(0, plane_coords);
          tfa1.setNormals(0, plane_normals1);
          Shape3D ps1 = new Shape3D();
          ps1.setGeometry(tfa1);
          ps1.setAppearance(createPlaneAppearance(PolygonAttributes.CULL_FRONT));

		  TriangleFanArray tfa2 = new TriangleFanArray(
            4, GeometryArray.COORDINATES | GeometryArray.NORMALS, four);
          tfa2.setCoordinates(0, plane_coords);
          tfa2.setNormals(0, plane_normals1);
          Shape3D ps2 = new Shape3D();
          ps2.setGeometry(tfa2);
          ps2.setAppearance(createPlaneAppearance(PolygonAttributes.CULL_BACK));

          TransformGroup stg = new TransformGroup();
          Transform3D st3d = new Transform3D();
          st3d.setTranslation(new Vector3d(0.0, 1.0-j*space, 0.0));
          stg.setTransform(st3d);
          stg.addChild(ps1);
          stg.addChild(ps2);
          stripe_bg.addChild(stg);
        }
      }

	  boolean add_coord = false;

      int[] sl;
      sl = new int[pointcnt];
      for (int i=0; i<pointcnt; i++) sl[i]=plane_cnt;
	  conn_lines = new LineStripArray(
        pointcnt*plane_cnt, GeometryArray.COORDINATES, sl);

      int[] tsl = { 2*pointcnt };
      for (int i=0; i<plane_cnt-1; i++)
      {
        TriangleStripArray tsa = new TriangleStripArray(
          2*pointcnt, GeometryArray.COORDINATES | GeometryArray.NORMALS , tsl);
        surface_v.addElement(tsa);
      }

      // create LineStripArray out of data
      for(int i=0; i<pointcnt; i++)
      {
        Vector act_point = (Vector) points_v.elementAt(i);

        for(int d=0; d<dimcnt; d+=2)
        {
          if(dimcnt % 2 == 1 && d+1==dimcnt) add_coord=true;

          Point3d p = new Point3d();
          p.x = ((Range)scaling_v.elementAt(d)).normalize(
	            ((Double)act_point.elementAt(d)).doubleValue());
	      p.y = 1.0 - ((double)(d/2))*space;
          if (add_coord == false)
            p.z = ((Range)scaling_v.elementAt(d+1)).normalize(
	              ((Double)act_point.elementAt(d+1)).doubleValue());
	      else p.z = p.x;
          ((LineStripArray)line_v.elementAt(d/2)).setCoordinate(i, p);
          if(style.showlines==true)
            conn_lines.setCoordinate(plane_cnt*i+(d/2), p);

          if(style.showsurface==true && d+2<dimcnt)
          {
            Point3d p1 = new Point3d();
            p1.x = ((Range)scaling_v.elementAt(d+2)).normalize(
	               ((Double)act_point.elementAt(d+2)).doubleValue());
            p1.y = 1.0 - ((double)((d+2)/2))*space;
            if(d+3<dimcnt)
              p1.z = ((Range)scaling_v.elementAt(d+3)).normalize(
	                 ((Double)act_point.elementAt(d+3)).doubleValue());
            else p1.z=p1.x;
            ((TriangleStripArray)surface_v.elementAt(d/2)).setCoordinate(2*i, p);
            ((TriangleStripArray)surface_v.elementAt(d/2)).setCoordinate(2*i+1, p1);
            if (i<pointcnt-1) // calc the normals (until the pre-last point)
            {
              Point3d p2 = new Point3d();
              Vector next_point = (Vector) points_v.elementAt(i+1);
              p2.x = ((Range)scaling_v.elementAt(d)).normalize(
	            ((Double)next_point.elementAt(d)).doubleValue());
	          p2.y = 1.0 - ((double)(d/2))*space;
              if (add_coord == false)
                p2.z = ((Range)scaling_v.elementAt(d+1)).normalize(
	              ((Double)next_point.elementAt(d+1)).doubleValue());
	          else p2.z = p2.x;

              Point3d p3 = new Point3d();
              p3.x = ((Range)scaling_v.elementAt(d+2)).normalize(
	               ((Double)next_point.elementAt(d+2)).doubleValue());
              p3.y = 1.0 - ((double)((d+2)/2))*space;
              if(d+3<dimcnt)
                p1.z = ((Range)scaling_v.elementAt(d+3)).normalize(
	                 ((Double)next_point.elementAt(d+3)).doubleValue());
              else p1.z=p1.x;

              Vector3f n1 = MathExt.calcNormal(p, p1, p2);
              Vector3f n2 = MathExt.calcNormal(p3, p, p1);

              ((TriangleStripArray)surface_v.elementAt(d/2)).setNormal(2*i, n1);
              ((TriangleStripArray)surface_v.elementAt(d/2)).setNormal(2*i+1, n2);
            }
          }
        } // for d
        add_coord=false;

      } // for i


      // add LineArrays to Shape3Ds
      for (int i=0; i<line_v.size(); i++)
      {
        Shape3D shape = new Shape3D();
        shape.setGeometry((LineStripArray) line_v.elementAt(i));

        // create the Appearance of style for all 2d-trajectories
        Appearance temp_app = style.getAppearance(2*i);
        Material temp_mat = temp_app.getMaterial();
        Color3f temp_col = new Color3f();
        temp_mat.getAmbientColor(temp_col);

	    Appearance app = new Appearance();
	    Color3f color = temp_col;
        ColoringAttributes ca = new ColoringAttributes();
        ca.setColor(color);
        app.setColoringAttributes(ca);

        LineAttributes lat = new LineAttributes();
        lat.setLineWidth(TRAJECTORY_THICKNESS);
        lat.setLineAntialiasingEnable(true);
        app.setLineAttributes(lat);

        shape.setAppearance(app);

        // add to lineShape vector
		lineShape_v.addElement(shape);
        stripe_bg.addChild(shape);
      }

      // add connection-lines if specified
      if(style.showlines==true)
      {
        Appearance app = new Appearance();
        ColoringAttributes ca = new ColoringAttributes();
        ca.setColor(new Color3f(0.45f, 0.45f, 0.45f));
        LineAttributes lat = new LineAttributes();
        lat.setLineAntialiasingEnable(true);
        app.setLineAttributes(lat);
        app.setColoringAttributes(ca);
        Shape3D shape = new Shape3D();
        shape.setGeometry(conn_lines);
        shape.setAppearance(app);
        stripe_bg.addChild(shape);
      }

      // surfaces, if specified
      if(style.showsurface==true)
      {
        for(int i=0; i<surface_v.size(); i++)
        {
          Shape3D shape = new Shape3D();
          shape.setGeometry((TriangleStripArray)surface_v.elementAt(i));

		  Appearance temp_app = style.getAppearance(2*i+1);
          Material temp_mat = temp_app.getMaterial();
          Color3f temp_col = new Color3f();
          temp_mat.getAmbientColor(temp_col);

          shape.setAppearance(
            createSurfaceAppearance(temp_col, style.sur_transparency));

          stripe_bg.addChild(shape);
        }
      }
	}

   /**
    * Inserts the text2d-elements specifying the coordinate's range
    * into the BranchGroup
    */
    private void createAnnotation(int dimcnt, Vector scaling_v)
	{
 	  TransformGroup tg, tga;
	  Transform3D t;
      Text2D txt;
      Vector3d[] pos = { new Vector3d(0.0, 0.03, 0.0),
                         new Vector3d(0.9, 0.03, 0.0),
                         new Vector3d(-0.15, -0.03, 0.0),
                         new Vector3d(-0.15, -1.0, 0.0) };
	  
      double space = 1.0;
	  int plane_cnt = (1+dimcnt)/2;
      if(plane_cnt>1)
        space = 1 / (double)(plane_cnt-1);
      else space = 1.0;

      tga = new TransformGroup();

      for(int i=0; i<plane_cnt; i++)
      {
		tg = new TransformGroup();
	    t = new Transform3D();
        Vector3d tl = new Vector3d(pos[0]);
        tl.z = 1.0 - ((double)i)*space;
	    t.setTranslation( tl );
	    tg.setTransform(t);
        String str1 = new String(""+((Range)scaling_v.elementAt(2*i)).getMin2());
	    txt = new Text2D(str1, new Color3f(0.9f, 1.0f, 1.0f), "System", 15, 1);
        tg.addChild(txt);
	    tga.addChild(tg);


		tg = new TransformGroup();
	    t = new Transform3D();
        tl = new Vector3d(pos[1]);
        tl.z = 1.0 - ((double)i)*space;
	    t.setTranslation( tl );
	    tg.setTransform(t);
        String str2 = new String(""+((Range)scaling_v.elementAt(2*i)).getMax2());
	    txt = new Text2D(str2, new Color3f(0.9f, 1.0f, 1.0f), "System", 15, 1);
        tg.addChild(txt);
	    tga.addChild(tg);

		tg = new TransformGroup();
	    t = new Transform3D();
        tl = new Vector3d(pos[2]);
        tl.z = 1.0 - ((double)i)*space;
	    t.setTranslation( tl );
	    tg.setTransform(t);
        if(2*i+1<dimcnt)
          str1 = new String(""+((Range)scaling_v.elementAt(2*i+1)).getMin2());
	    txt = new Text2D(str1, new Color3f(0.9f, 1.0f, 1.0f), "System", 15, 1);
        tg.addChild(txt);
	    tga.addChild(tg);

		tg = new TransformGroup();
	    t = new Transform3D();
        tl = new Vector3d(pos[3]);
        tl.z = 1.0 - ((double)i)*space;
	    t.setTranslation( tl );
	    tg.setTransform(t);
        if(2*i+1<dimcnt)
          str2 = new String(""+((Range)scaling_v.elementAt(2*i+1)).getMax2());
	    txt = new Text2D(str2, new Color3f(0.9f, 1.0f, 1.0f), "System", 15, 1);
        tg.addChild(txt);
	    tga.addChild(tg);
	  }

	  t = new Transform3D();
	  t.rotX(-Math.PI/2.0);
	  tga.setTransform(t);
	  addChild(tga);

	}

   /**
    * Inserts text3D-elements into the scene specifying the letters
    * for each variable.
    */
    private void createVariables(int dimcnt, Vector variablenames_v)
	{
  	  TransformGroup tg, tga;
	  Transform3D t;
      Text2D txt;

	  double space = 1.0;
	  int plane_cnt = (1+dimcnt)/2;
      if(plane_cnt>1)
        space = 1 / (double)(plane_cnt-1);
      else space = 1.0;

      tga = new TransformGroup();

      for(int i=0; i<plane_cnt; i++)
      {
        tg = new TransformGroup();
	    t = new Transform3D();
	    t.setTranslation( new Vector3d(0.47, 0.03, 1.0-((double)i)*space) );
	    tg.setTransform(t);
        String str = new String(""+((String)variablenames_v.elementAt(2*i)));
	    txt = new Text2D(str, new Color3f(0.7f, 0.8f, 1.0f), "System", 18, 1);
        tg.addChild(txt);
	    tga.addChild(tg);

        tg = new TransformGroup();
	    t = new Transform3D();
	    t.setTranslation( new Vector3d(-0.1, -0.53, 1.0-((double)i)*space) );
	    tg.setTransform(t);
        if(2*i+1<dimcnt)
          str = ""+((String)variablenames_v.elementAt(2*i+1));
	    txt = new Text2D(str, new Color3f(0.7f, 0.8f, 1.0f), "System", 18, 1);
        tg.addChild(txt);
	    tga.addChild(tg);
      }

	  t = new Transform3D();
	  t.rotX(-Math.PI/2.0);
	  tga.setTransform(t);
	  addChild(tga);


	}

   /**
    * Inserts the spheres, which are dragable with the mouse to change
    * the starting value of each variable
    */
    private void createController(Vector points_v, Vector scaling_v)
    {
      // appearance for the start-value marking sphere
      Appearance app = new Appearance();
      Color3f objColor = new Color3f(1.0f, 0.2f, 0.0f);

      Material mat = new Material(objColor, new Color3f(0.0f,0.0f,0.0f), objColor,
	                                              new Color3f(1.0f,1.0f,1.0f), 80.0f);
      app.setMaterial(mat);

	  double space = 1.0;
	  int plane_cnt = (1+dimcnt)/2;
      if(plane_cnt>1)
        space = 1 / (double)(plane_cnt-1);
      else space = 1.0;


      for (int i=0; i<plane_cnt;i++)
      {
        BranchGroup bg = new BranchGroup();
        addChild(bg);

        TransformGroup tg = new TransformGroup();
         Transform3D t3d = new Transform3D();

		 double x = ((Range)scaling_v.elementAt(i*2)).normalize(
		               ((Double)((Vector)points_v.elementAt(0)).elementAt(i*2)).doubleValue());
         double y = 0.0;
         if(2*i+1<dimcnt)
           y = ((Range)scaling_v.elementAt(i*2+1)).normalize(
		       ((Double)((Vector)points_v.elementAt(0)).elementAt(i*2+1)).doubleValue());
         else y = x;

         t3d.setTranslation(new Vector3d(x, 1.0-((double)i)*space, y) );

         tg.setTransform(t3d);
         tg.setCapability(Node.ENABLE_PICK_REPORTING);
         tg.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
         tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        bg.addChild(tg);

        tg.addChild((new Sphere(0.015f, app)));

        PickTranslateBehavior4 mt = new PickTranslateBehavior4(bg,
                                                parent.canvas,
                                                new BoundingSphere(new Point3d(0.0, 1.0, 1.0), 0.015),
                                                PickObject.USE_BOUNDS,
                                                i,
                                                1.0-((double)i)*space,
                                                parent);
        bg.addChild(mt);

      }


    }


   /**
    * By calling this constructor the graph is created using the specified
    * data.
    */
    public TDCGraph(GraphOutput parent, int dimcnt, int pointcnt,
	                Vector points_v, Vector scaling_v, Vector variablenames_v,
	                Style style)
	{
	  this.parent = parent;
	  this.style = style;
	  this.dimcnt=dimcnt;
	  this.pointcnt=pointcnt;
	  
      stripe_bg = new BranchGroup();
      stripe_bg.setCapability(BranchGroup.ALLOW_DETACH);

	  createGeometries(dimcnt, pointcnt, points_v, scaling_v);
      createVariables(dimcnt, variablenames_v);
      createController(points_v, scaling_v);

      setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
      setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);

      if(style.showscaling==true) createAnnotation(dimcnt, scaling_v);

	  if (style.showlines==true && dimcnt>1)
	  {
	    Appearance app1 = new Appearance();
	    Color3f color1 = new Color3f(1.0f, 1.0f, 1.0f);
	    ColoringAttributes ca1 = new ColoringAttributes();
	    ca1.setColor(color1);
	    app1.setColoringAttributes(ca1);

	    PolygonAttributes pa1 = new PolygonAttributes();
	    pa1.setPolygonMode(pa1.POLYGON_LINE);
	    pa1.setCullFace(pa1.CULL_NONE);
	    app1.setPolygonAttributes(pa1);
	  }
	  setCapability(BranchGroup.ALLOW_DETACH);
      addChild(stripe_bg);
	  compile();

	}

  /**
   * creates the appearance for a plane with the specified
   * CULL_FACE
   */
 private Appearance createPlaneAppearance(int cullface)
 {
   Appearance app1 = new Appearance();
   Material mat = new Material(new Color3f(0.1f,0.1f,0.1f),
                               new Color3f(0.3f,0.3f,0.3f),
                               new Color3f(0.3f,0.3f,0.3f),
                               new Color3f(0.1f,0.1f,0.1f),
                               20.0f);
   app1.setMaterial(mat);
   PolygonAttributes pa = new PolygonAttributes();
   pa.setCullFace(cullface);
   app1.setPolygonAttributes(pa);

   TransparencyAttributes ta = new TransparencyAttributes();
   ta.setTransparencyMode(ta.BLENDED);
   ta.setTransparency(PLANE_TRANSPARENCY);
   app1.setTransparencyAttributes(ta);
   return app1;
  }

  /**
   * creates the appearance for a surface with the specified
   * color an transparency
   */
  private Appearance createSurfaceAppearance(Color3f color, float transparency)
  {
    Appearance app = new Appearance();

    Material mat = new Material(color,
                                new Color3f(0.0f,0.0f,0.0f),
                                color,
                                new Color3f(1.0f,1.0f,1.0f),
                                80.0f);
    mat.setCapability(Material.ALLOW_COMPONENT_READ);
    app.setMaterial(mat);
    app.setCapability(Appearance.ALLOW_MATERIAL_READ);
    app.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
    PolygonAttributes pa = new PolygonAttributes();

    TransparencyAttributes ta = new TransparencyAttributes();
    ta.setTransparencyMode(ta.BLENDED);
    ta.setTransparency(transparency);
    ta.setCapability(TransparencyAttributes.ALLOW_VALUE_READ);
    app.setTransparencyAttributes(ta);

    pa.setCullFace(pa.CULL_NONE);
    app.setPolygonAttributes(pa);

    return app;
  }



} // end of class TDCGraph
